@@ -39,7 +39,8 @@ from libqtile.command.base import CommandObject, expose_command
 from libqtile.log_utils import logger
 
 if TYPE_CHECKING:
-    from typing import Any, Callable, Iterable
+    from collections.abc import Callable, Iterable
+    from typing import Any
 
     from libqtile.backend import base
     from libqtile.bar import BarType
@@ -89,7 +90,7 @@ class Key:
         self.swallow = swallow
 
     def __repr__(self) -> str:
-        return "<Key (%s, %s)>" % (self.modifiers, self.key)
+        return f"<Key ({self.modifiers}, {self.key})>"
 
 
 class KeyChord:
@@ -154,7 +155,7 @@ class KeyChord:
         self.swallow = swallow
 
     def __repr__(self) -> str:
-        return "<KeyChord (%s, %s)>" % (self.modifiers, self.key)
+        return f"<KeyChord ({self.modifiers}, {self.key})>"
 
 
 class Mouse:
@@ -204,7 +205,7 @@ class Drag(Mouse):
         self.warp_pointer = warp_pointer
 
     def __repr__(self) -> str:
-        return "<Drag (%s, %s)>" % (self.modifiers, self.button)
+        return f"<Drag ({self.modifiers}, {self.button})>"
 
 
 class Click(Mouse):
@@ -218,14 +219,14 @@ class Click(Mouse):
         ``"shift"``, ``"lock"``, ``"control"``, ``"mod1"``, ``"mod2"``, ``"mod3"``,
         ``"mod4"``, ``"mod5"``.
     button:
-        The button used to start dragging e.g. ``"Button1"``.
+        The button used to click e.g. ``"Button1"``.
     commands:
-        A list :class:`LazyCall` objects to evaluate in sequence upon drag.
+        A list :class:`LazyCall` objects to evaluate in sequence upon click.
 
     """
 
     def __repr__(self) -> str:
-        return "<Click (%s, %s)>" % (self.modifiers, self.button)
+        return f"<Click ({self.modifiers}, {self.button})>"
 
 
 class EzConfig:
@@ -274,7 +275,7 @@ class EzConfig:
             raise utils.QtileError(msg % spec)
 
         if len(keys) > 1:
-            msg = "Key chains are not supported: %s" % spec
+            msg = f"Key chains are not supported: {spec}"
             raise utils.QtileError(msg)
 
         return mods, keys[0]
@@ -351,7 +352,7 @@ class EzClick(EzConfig, Click):
 
     def __init__(self, btndef: str, *commands: LazyCall) -> None:
         modkeys, button = self.parse(btndef)
-        button = "Button%s" % button
+        button = f"Button{button}"
         super().__init__(modkeys, button, *commands)
 
 
@@ -372,7 +373,7 @@ class EzDrag(EzConfig, Drag):
 
     def __init__(self, btndef: str, *commands: LazyCall, start: LazyCall | None = None) -> None:
         modkeys, button = self.parse(btndef)
-        button = "Button%s" % button
+        button = f"Button{button}"
         super().__init__(modkeys, button, *commands, start=start)
 
 
@@ -456,6 +457,42 @@ class Screen(CommandObject):
         self.height = height if height is not None else 0
         self.previous_group: _Group | None = None
 
+    def __eq__(self, other: object) -> bool:
+        # When we trigger a reconfigure_screens(), _process_screens()
+        # re-creates the Screen() objects. Since we only replace qtile.screens,
+        # groups, widgets, etc. which have saved a reference to the Screen
+        # object will cause comparisons to fail even if a Screen object after
+        # reconfiguration represents the same geometry area, which is generally
+        # not what people expect.
+        #
+        # For example, this happens here:
+        # https://github.com/qtile/qtile/blob/42e03cdb6bbb6d88f0fd58927c6ec6258f625961/libqtile/group.py#L167-L168
+        # which does:
+        #
+        #     if self.current_window and self.screen == self.qtile.current_screen:
+        #         self.current_window.focus(warp)
+        #
+        # This will fail, because the group's object is stale, and we will not
+        # focus things correctly. This is one example, but there are several
+        # other object trees that save copies of the current screen.
+        #
+        # One problem with this approach is: if the reconfigure_screens() event
+        # comes as a result of a geometry change, this comparison will still
+        # fail. To fix this, we need to uniquely identify each Screen, which we
+        # can do via EDID info in some future work. For now, this makes things
+        # better than it was :)
+        if not isinstance(other, Screen):
+            return False
+        return (
+            other.x == self.x
+            and other.y == self.y
+            and other.width == self.width
+            and other.height == self.height
+        )
+
+    def __hash__(self) -> int:
+        return hash((self.x, self.y, self.width, self.height))
+
     def _configure(
         self,
         qtile: Qtile,
@@ -492,6 +529,16 @@ class Screen(CommandObject):
     def gaps(self) -> Iterable[BarType]:
         return (i for i in [self.top, self.bottom, self.left, self.right] if i)
 
+    def finalize_gaps(self) -> None:
+        def remove(attr: str) -> None:
+            gap = getattr(self, attr, None)
+            if gap is not None:
+                setattr(self, attr, None)
+                gap.finalize()
+
+        for attr in ["top", "bottom", "left", "right"]:
+            remove(attr)
+
     @property
     def dx(self) -> int:
         if self.left and getattr(self.left, "reserve", True):
@@ -655,20 +702,20 @@ class Screen(CommandObject):
         return dict(index=self.index, width=self.width, height=self.height, x=self.x, y=self.y)
 
     @expose_command()
-    def next_group(self, skip_empty: bool = False, skip_managed: bool = False) -> None:
+    def next_group(
+        self, skip_empty: bool = False, skip_managed: bool = False, warp: bool = True
+    ) -> None:
         """Switch to the next group"""
-        n = self.group.get_next_group(skip_empty, skip_managed)
-        self.set_group(n)
-        return n.name
+        group = self.group.get_next_group(skip_empty, skip_managed)
+        self.set_group(group, warp=warp)
 
     @expose_command()
     def prev_group(
         self, skip_empty: bool = False, skip_managed: bool = False, warp: bool = True
     ) -> None:
         """Switch to the previous group"""
-        n = self.group.get_previous_group(skip_empty, skip_managed)
-        self.set_group(n, warp=warp)
-        return n.name
+        group = self.group.get_previous_group(skip_empty, skip_managed)
+        self.set_group(group, warp=warp)
 
     @expose_command()
     def toggle_group(self, group_name: str | None = None, warp: bool = True) -> None:
@@ -769,7 +816,7 @@ class Group:
                 "screen_affinity",
             ],
         )
-        return "<config.Group %r (%s)>" % (self.name, attrs)
+        return f"<config.Group {self.name!r} ({attrs})>"
 
 
 class ScratchPad(Group):
@@ -816,7 +863,7 @@ class ScratchPad(Group):
         self.single = single
 
     def __repr__(self) -> str:
-        return "<config.ScratchPad %r (%s)>" % (
+        return "<config.ScratchPad {!r} ({})>".format(
             self.name,
             ", ".join(dd.name for dd in self.dropdowns),
         )
@@ -874,7 +921,7 @@ class InvertMatch(_Match):
         return not self.match.compare(client)
 
     def __repr__(self) -> str:
-        return "<InvertMatch(%r)>" % self.match
+        return f"<InvertMatch({self.match!r})>"
 
 
 class MatchAll(_Match):
@@ -887,7 +934,7 @@ class MatchAll(_Match):
         return all(m.compare(client) for m in self.matches)
 
     def __repr__(self) -> str:
-        return "<MatchAll(%r)>" % (self.matches,)
+        return f"<MatchAll({self.matches!r})>"
 
 
 class MatchAny(MatchAll):
@@ -897,7 +944,7 @@ class MatchAny(MatchAll):
         return any(m.compare(client) for m in self.matches)
 
     def __repr__(self) -> str:
-        return "<MatchAny(%r)>" % (self.matches,)
+        return f"<MatchAny({self.matches!r})>"
 
 
 class MatchOnlyOne(_Match):
@@ -911,7 +958,7 @@ class MatchOnlyOne(_Match):
         return self.match1.compare(client) != self.match2.compare(client)
 
     def __repr__(self) -> str:
-        return "<MatchOnlyOne(%r, %r)>" % (self.match1, self.match2)
+        return f"<MatchOnlyOne({self.match1!r}, {self.match2!r})>"
 
 
 class Match(_Match):
@@ -983,7 +1030,7 @@ class Match(_Match):
             try:
                 self._rules["net_wm_pid"] = int(net_wm_pid)
             except ValueError:
-                error = 'Invalid rule for net_wm_pid: "%s" only int allowed' % str(net_wm_pid)
+                error = f'Invalid rule for net_wm_pid: "{str(net_wm_pid)}" only int allowed'
                 raise utils.QtileError(error)
         if func is not None:
             self._rules["func"] = func
@@ -1059,7 +1106,7 @@ class Match(_Match):
                 callback(c)
 
     def __repr__(self) -> str:
-        return "<Match %s>" % self._rules
+        return f"<Match {self._rules}>"
 
 
 class Rule:
@@ -1079,6 +1126,8 @@ class Rule:
         Should we override the group's exclusive setting?
     break_on_match:
         Should we stop applying rules if this rule is matched?
+    one_time:
+        Should we delete rule once if it has been matched?
 
     """
 
@@ -1089,6 +1138,7 @@ class Rule:
         float: bool = False,
         intrusive: bool = False,
         break_on_match: bool = True,
+        one_time: bool = False,
     ) -> None:
         if isinstance(match, _Match):
             self.matchlist = [match]
@@ -1098,6 +1148,7 @@ class Rule:
         self.float = float
         self.intrusive = intrusive
         self.break_on_match = break_on_match
+        self.one_time = one_time
 
     def matches(self, w: base.Window) -> bool:
         return any(w.match(m) for m in self.matchlist)
@@ -1106,7 +1157,7 @@ class Rule:
         actions = utils.describe_attributes(
             self, ["group", "float", "intrusive", "break_on_match"]
         )
-        return "<Rule match=%r actions=(%s)>" % (self.matchlist, actions)
+        return f"<Rule match={self.matchlist!r} actions=({actions})>"
 
 
 class DropDown(configurable.Configurable):
